﻿using Autofac;
using Autofac.Configuration;
using Autofac.Integration.Mvc;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Web;
using System.Web.Mvc;

namespace MvcSettings
{
    public interface IDependency { }
    public interface ISingleton { }
    public interface IPerWebRequest { }

    public static class AutofacConfig
    {
        public static void RegisterDependencies()
        {
            var builder = new ContainerBuilder();

            // autowire
            builder.RegisterAssemblyTypes(Assembly.GetExecutingAssembly())
                    .As<IDependency>()
                    .AsImplementedInterfaces()
                    .InstancePerDependency();

            builder.RegisterAssemblyTypes(Assembly.GetExecutingAssembly())
                    .As<ISingleton>()
                    .AsImplementedInterfaces()
                    .SingleInstance();

            builder.RegisterAssemblyTypes(Assembly.GetExecutingAssembly())
                    .As<IPerWebRequest>()
                    .AsImplementedInterfaces()
                    .InstancePerRequest();

            // make controllers use constructor injection
            builder.RegisterControllers(Assembly.GetExecutingAssembly());

            // register modules
            builder.RegisterModule(new ConfigurationSettingsReader("autofac"));
            
            DependencyResolver.SetResolver(new AutofacDependencyResolver(builder.Build())); 
        }
    }
}