﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Web;
using System.Web.Caching;

namespace MvcSettings.Caching
{
    public class AspNetCache : ICache
    {
        // max is a year
        const int _maxDurationSeconds = 31536000;

        /// <summary>
        /// The maximum duration the item can be stored in cache
        /// </summary>
        public int MaxDuration
        {
            get
            {
                return _maxDurationSeconds;
            }
        }

        public IEnumerable<string> Keys
        {
            get 
            {
                var keys = new List<string>();
                var enumerator = HttpRuntime.Cache.GetEnumerator();
                while (enumerator.MoveNext())
                {
                    keys.Add(enumerator.Key.ToString());
                }
                return keys;
            }
        }

        public IDictionary<string, object> Values
        {
            get
            {
                var values = new Dictionary<string, object>();
                var enumerator = HttpRuntime.Cache.GetEnumerator();
                while (enumerator.MoveNext())
                {
                    values.Add(enumerator.Key.ToString(), enumerator.Value);
                }
                return values;
            }
        }

        public object Get(string key)
        {
            return HttpRuntime.Cache.Get(key);
        }

        public void Store(string key, object value, bool sliding, int secondsToCache)
        {
            if (secondsToCache <= 0 || secondsToCache > _maxDurationSeconds)
            {
                throw new ArgumentOutOfRangeException(string.Format("secondsToCache must be greater than 0 and less than or equal to {0}", _maxDurationSeconds));
            }

            var ts = TimeSpan.FromSeconds(int.MaxValue);
            var ts2 = TimeSpan.FromSeconds(10);

            HttpRuntime.Cache.Insert(
                        key: key,
                        value: value,
                        dependencies: null,
                        absoluteExpiration: sliding == false ? DateTime.UtcNow.AddSeconds(secondsToCache) : Cache.NoAbsoluteExpiration,
                        slidingExpiration: sliding == true ? TimeSpan.FromSeconds(secondsToCache) : Cache.NoSlidingExpiration,
                        priority: CacheItemPriority.Default,
                        onRemoveCallback: ItemRemoved
                        );
        }

        public void Remove(string key)
        {
            HttpRuntime.Cache.Remove(key);
        }

        public void Clear()
        {
            var enumerator = HttpRuntime.Cache.GetEnumerator();
            while (enumerator.MoveNext())
            {
                string key = enumerator.Key.ToString();
                HttpRuntime.Cache.Remove(key);
            }
        }

        private void ItemRemoved(string key, object value, CacheItemRemovedReason reason)
        {
        }
    }
}