﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Routing;

namespace MvcSettings.Caching
{
    public class OutputCacheItem
    {
        public string CacheKey { get; private set; }
        public string ContentType { get; private set; }
        public string Content { get; private set; }
        public string ControllerName { get; private set; }
        public string ActionName { get; private set; }
        public RouteValueDictionary RouteValues { get; private set; }
        public bool Sliding { get; private set; }
        public DateTime Created { get; private set; }
        public DateTime Accessed { get; private set; }
        public int CacheForSeconds { get; private set; }
        public DateTime Expires { get; private set; }

        public OutputCacheItem(string cacheKey, string contentType, string content, 
            string controllerName, string actionName, string areaName, IDictionary<string, object> actionParameters,
            bool sliding, int cacheForSeconds)
        {
            if (string.IsNullOrEmpty(cacheKey))
            {
                throw new ArgumentNullException("cacheKey");
            }

            if (string.IsNullOrEmpty(contentType))
            {
                throw new ArgumentNullException("contentType");
            }

            if (string.IsNullOrEmpty(content))
            {
                throw new ArgumentNullException("content");
            }

            if (string.IsNullOrEmpty(controllerName))
            {
                throw new ArgumentNullException("controller");
            }

            if (string.IsNullOrEmpty(actionName))
            {
                throw new ArgumentNullException("action");
            }

            CacheKey = cacheKey;
            Created = DateTime.UtcNow;
            Accessed = Created;
            ContentType = contentType;
            Content = content;
            ControllerName = controllerName;
            ActionName = actionName;
            Sliding = sliding;
            CacheForSeconds = cacheForSeconds;
            Expires = Created.AddSeconds(CacheForSeconds);
            RouteValues = new RouteValueDictionary(); 

            if (actionParameters != null)
            {
                RouteValues = new RouteValueDictionary(actionParameters);
            }

            if (!string.IsNullOrEmpty(areaName))
            {
                RouteValues["area"] = areaName;
            }
        }

        public bool BelongsToController(string controller)
        {
            if (ControllerName.Equals(controller, StringComparison.CurrentCultureIgnoreCase) && !RouteValues.ContainsKey("area"))
            {
                return true;
            }
            return false;
        }

        public bool BelongsToAction(string controller, string action)
        {
            if (ControllerName.Equals(controller, StringComparison.CurrentCultureIgnoreCase) &&
                ActionName.Equals(action, StringComparison.CurrentCultureIgnoreCase) && !RouteValues.ContainsKey("area"))
            {
                return true;
            }
            return false;
        }

        public void UpdateAccess()
        {
            Accessed = DateTime.UtcNow;
            if (Sliding)
            {
                Expires = Accessed.AddSeconds(CacheForSeconds);
            }
        }
    }
}