﻿using MvcSettings.Caching;
using MvcSettings.Services;
using MvcSettings.ViewModels;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace MvcSettings.Controllers
{
    [RoutePrefix("cache")]
    public class CacheController : Controller
    {
        private readonly IOutputCacheManager _outputCacheManager;
        public CacheController(IOutputCacheManager outputCacheManager)
        {
            _outputCacheManager = outputCacheManager;
        }

        [Route("")]
        public ActionResult Index()
        {
            var viewModel = new OutputCacheViewModel();
            viewModel.Items = _outputCacheManager.CacheItems;

            return View(viewModel);
        }

        [Route("test")]
        public ActionResult Test()
        {
            _outputCacheManager.Clear("Test", "Home", new { message = "", count = 1 });
            return RedirectToAction("Index");
        }

        [Route("evict/{cacheKey}")]
        public ActionResult Evict(string cacheKey)
        {
            _outputCacheManager.Remove(cacheKey);
            return RedirectToAction("Index");
        }

        [Route("evictcontroller/{controllerName}")]
        public ActionResult EvictController(string controllerName)
        {
            _outputCacheManager.Clear(controllerName);
            return RedirectToAction("Index");
        }

        [Route("evictaction/{controllerName}/{actionName}")]
        public ActionResult EvictAction(string controllerName, string actionName)
        {
            _outputCacheManager.Clear(actionName, controllerName);
            return RedirectToAction("Index");
        }
	}
}