﻿using MvcSettings.Caching;
using MvcSettings.Services;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;

namespace MvcSettings.Controllers
{
    public class HomeController : Controller
    {
        private readonly ISettings _settings;
        private readonly ICache _cache;

        public HomeController(ISettings settings, ICache cache)
        {
            _settings = settings;
            _cache = cache;
        }

        public ActionResult Index()
        {
            return RedirectToAction("Index", "Cache");
        }

        [CacheOnServer(Sliding=true, Duration=60), Route("sliding")]
        public ActionResult Sliding()
        {
            return Content(string.Format("Sliding: {0}", DateTime.UtcNow));
        }

        [CacheOnServer(Sliding = false, Duration = 60), Route("absolute")]
        public ActionResult Absolute()
        {
            return Content(string.Format("Absolute: {0}", DateTime.UtcNow));
        }

        [CacheOnServer()]
        [Route("test")]
        public ActionResult Test(string message, int count)
        {
            return Content(string.Format("{0} at {1} count {2}", message, DateTime.UtcNow, count));
        }

        public ActionResult Save()
        {
            using (var uow = new UnitOfWork())
            {
                var settings = new Settings(uow);
                settings.General.SiteName = "Talk Sharp 2";
                settings.Seo.HomeMetaDescription = "Welcome to Talk Sharp";
                settings.Save();

                var settings2 = new Settings(uow);
                string output = string.Format("SiteName: {0} HomeMetaDescription {1}",
                                                settings2.General.SiteName,
                                                settings2.Seo.HomeMetaDescription
                                                );
                return Content(output);
            }
        }

        public ActionResult SaveIoC()
        {
            _settings.General.SiteName = "Talk Sharp IoC";
            _settings.Seo.HomeMetaDescription = "Welcome to Talk Sharp with IoC";
            _settings.Save();

            string output = string.Format("SiteName: {0} HomeMetaDescription {1}",
                                _settings.General.SiteName,
                                _settings.Seo.HomeMetaDescription
                                );

            return Content(output);   
        }
    }
}